(* ::Package:: *)

% Written by Dr. Gregory Scott from Imperial College. Academic Clinical Lecturer, The Computational, Cognitive and Clinical Neuroimaging Laboratory (C3NL) Faculty of Medicine, Department of Medicine
% https://www.imperial.ac.uk/people/gregory.scott99
% 1D BDM implementation of method as described in:
% A Decomposition Method for Global Evaluation of Shannon Entropy and Local Estimations of Algorithmic Complexity
% By Hector Zenil, Santiago Hern\[AAcute]ndez-Orozco, Narsis A. Kiani, Fernando Soler-Toscano, Antonio Rueda-Toicen
% preprint available at https://arxiv.org/abs/1609.00110

function bdm = stringbdm(str, len)
if nargin < 2; len = 1; end

persistent HashStrings % #ok<PSET>
if(isempty(HashStrings)); load('HashStrings.mat', 'HashStrings'); end

if(length(str) > 12)
    parts = stringpartition(str, 12, len);
else
    parts = {str};
end

tal = tally(parts);
bdm = sum(cellfun(@(a,b) HashStrings{a,:} + log2(b), tal(:,1), tal(:,2)));
end

function parts = stringpartition(str, n, d)
parts = arrayfun(@(l) str(l:l+(n-1)), 1:d:(length(str)-(n-1)), ...
    'UniformOutput', false);
end

function tal = tally(x)
tab = tabulate(x); tal = tab(:,[1,2]);
end
